from __future__ import unicode_literals

try:
    # python 2
    from urlparse import urljoin
except ImportError:
    # python 3
    from urllib.parse import urljoin

from django.utils.translation import ugettext as _

from ..utils.configuration import get_required_setting
from ..utils.emails import umanage_send_email


def send_change_email_notice_email(to_user,
                                   authorization,
                                   markdown_template='umanage/emails/change_email_requested.md',
                                   **kwargs):
    """Sends an email to a user alerting them that their email has been
    requested to change.
    """
    context = _get_change_email_context(to_user, authorization)
    return umanage_send_email(
        to_user=to_user,
        subject=_('Email Change Requested'),
        markdown_template=markdown_template,
        context=context,
        **kwargs)


def send_change_email_activation_email(to_user,
                                       authorization,
                                       markdown_template='umanage/emails/change_email_activation.md',
                                       **kwargs):
    """Sends an email to the address of the newly provided email for
    activation.
    """
    context = _get_change_email_context(to_user, authorization)
    url = urljoin(context.get('site_root_uri'),
                  authorization.get_absolute_url())
    context['umanage_change_email_activation_url'] = url

    return umanage_send_email(
        to_user=to_user,
        subject=_('Activate New Email Address'),
        markdown_template=markdown_template,
        context=context,
        **kwargs)


def _get_change_email_context(to_user, authorization):
    """Common context for the change email flow."""
    context = {
        'new_email': authorization.email_address,
        'site_root_uri': get_required_setting('UMANAGE_SITE_ROOT_URI')
    }

    return context



from django.conf.urls import patterns
from django.conf.urls import url

from .views import ChangeEmailActivationView
from .views import ChangeEmailExpiredView
from .views import ChangeEmailSentView
from .views import ChangeEmailSuccessView
from .views import ChangeEmailView


urlpatterns = patterns('',
    url(r'^/change-email/?$', ChangeEmailView.as_view(), name='umanage_change_email'),
    url(r'^/change-email/expired/?$', ChangeEmailExpiredView.as_view(), name='umanage_change_email_expired'),
    url(r'^/change-email/sent/?$', ChangeEmailSentView.as_view(), name='umanage_change_email_sent'),
    url(r'^/change-email/success/?$', ChangeEmailSuccessView.as_view(), name='umanage_change_email_success'),
    url(r'^/change-email/(?P<authorization_token>(\w|-)+)/?$', ChangeEmailActivationView.as_view(), name='umanage_change_email_activation'),
)



from __future__ import unicode_literals

from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.views.generic.base import TemplateView
from django.views.generic.edit import FormView
from django_core.utils.loading import get_class_from_settings_full_path
from django_core.views.mixins.auth import LoginRequiredViewMixin

from ..exceptions import UManageSettingImproperlyConfigured
from .forms import UserAccountForm


class AccountView(LoginRequiredViewMixin, TemplateView):

    template_name = 'umanage/accounts/account_view.html'

    def get_context_data(self, **kwargs):
        context = super(AccountView, self).get_context_data(**kwargs)
        user = self.request.user
        settings_key = 'UMANAGE_USER_ACCOUNT_DISPLAY_FIELDS'
        user_fields_to_display = getattr(settings,
                                         settings_key,
                                         ('first_name', 'last_name', 'email'))

        if not isinstance(user_fields_to_display, tuple):
            raise UManageSettingImproperlyConfigured(settings_key)

        fields_to_display = []

        for field_name in user_fields_to_display:
            try:
                val = getattr(user, field_name)
                label = user._meta.get_field_by_name(field_name)[0].verbose_name
            except:
                raise UManageSettingImproperlyConfigured(
                    settings_key,
                    message=_('"{0}" is not a valid field on the User model. '
                              'Check the "{1}" config '
                              'setting.').format(field_name, settings_key)
                )
            fields_to_display.append((label.title(), val))

        context['fields_to_display'] = fields_to_display
        return context


class AccountEditView(LoginRequiredViewMixin, FormView):

    template_name = 'umanage/accounts/account_edit.html'
    form_class = UserAccountForm

    def dispatch(self, *args, **kwargs):
        settings_key = 'UMANAGE_USER_ACCOUNT_EDIT_FORM'
        if hasattr(settings, settings_key):
            try:
                self.form_class = get_class_from_settings_full_path(settings_key)
            except:
                msg = _('{0} setting path is either incorrect or the app is '
                        'not installed.  Please check the '
                        'configuration.').format(settings_key)
                raise UManageSettingImproperlyConfigured(settings_key, msg)

        return super(AccountEditView, self).dispatch(*args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(AccountEditView, self).get_form_kwargs()
        kwargs['instance'] = self.request.user
        return kwargs

    def form_valid(self, form):
        form.save()
        return super(AccountEditView, self).form_valid(form)

    def get_success_url(self):
        return reverse('umanage_account_view')



from __future__ import unicode_literals

from django.contrib.auth.forms import PasswordChangeForm
from django_core.utils.validators import validate_password_strength


class UManagePasswordChangeForm(PasswordChangeForm):

    def __init__(self, *args, **kwargs):
        super(UManagePasswordChangeForm, self).__init__(*args, **kwargs)
        self.fields['new_password1'].validators.append(validate_password_strength)



from __future__ import unicode_literals

from django import forms
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django_core.forms.widgets import ReadonlyWidget


class UserAccountForm(forms.ModelForm):
    """Form for editing a user account."""
    email = forms.EmailField(label=_('Email Address'),
                             widget=ReadonlyWidget,
                             required=False)

    class Meta:
        model = get_user_model()
        fields = ('first_name', 'last_name', 'email')

    def __init__(self, *args, **kwargs):
        super(UserAccountForm, self).__init__(*args, **kwargs)
        self.fields['email'].help_text = '<a href="{0}">{1}</a>'.format(
            reverse('umanage_change_email'),
            _('change')
        )

    def clean_email(self):
        """Email is readonly."""
        return self.instance.email



from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext as _


class UManageRequiredSettingImproperlyConfigured(ImproperlyConfigured):
    """UManange exceptions for not having a correct settings key."""

    def __init__(self, settings_key=None, message=None, *args, **kwargs):

        if not message and settings_key is not None:
            message = _('No setting found for "{0}".  This is a required '
                        'setting for the django-umanage app.'.format(
                                                                settings_key))

        super(UManageRequiredSettingImproperlyConfigured, self).__init__(
            message,
            *args,
            **kwargs
        )


class UManageSettingImproperlyConfigured(ImproperlyConfigured):
    """UManange exceptions for not having a correct settings key."""

    def __init__(self, settings_key=None, message=None, *args, **kwargs):

        if not message and settings_key is not None:
            message = _('"{0}" setting is set incorrectly.'.format(
                settings_key
            ))

        super(UManageSettingImproperlyConfigured, self).__init__(
            message,
            *args,
            **kwargs
        )



from __future__ import unicode_literals

try:
    # python 2
    from urlparse import urljoin
except ImportError:
    # python 3
    from urllib.parse import urljoin

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _

from ..utils.configuration import get_required_setting
from ..utils.emails import umanage_send_email


def send_forgot_password_email(to_user,
                               authorization,
                               markdown_template='umanage/emails/forgot_password.md',
                               **kwargs):
    """Sends an email to a user to reset their password.

    :param to_user: the user the email is for
    :param authorization: the ForgotPasswordAuthorization object to complete
        the flow.
    """
    context = _get_forgot_password_context(to_user)
    context['forgot_password_url'] = urljoin(
        context.get('site_root_uri'),
        reverse('umanage_forgot_password')
    )
    context['umanage_forgot_password_change_password_url'] = urljoin(
        context.get('site_root_uri'),
        authorization.get_absolute_url()
    )
    return umanage_send_email(
        to_user=to_user,
        subject=_('Password Reset'),
        markdown_template=markdown_template,
        context=context,
        **kwargs)


def _get_forgot_password_context(to_user):
    """Common context for the forgot password flow."""
    context = {
        'site_root_uri': get_required_setting('UMANAGE_SITE_ROOT_URI')
    }

    return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_core', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AccountActivationAuthorization',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('django_core.tokenauthorization',),
        ),
        migrations.CreateModel(
            name='ChangeEmailAuthorization',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('django_core.tokenauthorization',),
        ),
        migrations.CreateModel(
            name='ForgotPasswordAuthorization',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('django_core.tokenauthorization',),
        ),
    ]



from django.core.urlresolvers import reverse
from django.shortcuts import redirect


class UsablePasswordRequiredMiddleware(object):
    """Middleware for requiring a user to have a usable password.

    If a user authenticates through a 3rd party, then django sets the password
    to an unusable password.  If this is the case, the user needs to set a
    password.
    """
    def process_request(self, request):
        user = request.user
        set_password_url = reverse('umanage_set_password')
        is_request_set_password_url = set_password_url in request.path

        if (user.is_authenticated() and
            not user.has_usable_password() and
            not is_request_set_password_url):

            # redirect user to set password
            return redirect(set_password_url)



from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext as _
from django_core.forms.mixins.users import UserAuthorizationRequiredForm
from django_core.forms.widgets import ReadonlyWidget
from umanage.change_email.emails import send_change_email_activation_email
from umanage.change_email.emails import send_change_email_notice_email
from umanage.models import ChangeEmailAuthorization


class ChangeEmailForm(UserAuthorizationRequiredForm):
    """Form for changing a user's email."""
    error_messages = dict(UserAuthorizationRequiredForm.error_messages, **{
        'invalid_confirm_email': _("The new email and new email confirm must "
                                   "match exactly. Please enter it again."),
    })
    current_email = forms.EmailField(label=_('Current Email'),
                                     max_length=100, widget=ReadonlyWidget,
                                     required=False)
    new_email = forms.EmailField(label=_('New Email'))
    new_email_confirm = forms.EmailField(label=_('New Email Confirm'))

    def __init__(self, *args, **kwargs):
        super(ChangeEmailForm, self).__init__(*args, **kwargs)

        if 'current_email' not in self.initial:
            self.initial['current_email'] = self.user.email

        self.fields.keyOrder = ('current_email', 'password', 'new_email',
                                'new_email_confirm')

    def clean_new_email_confirm(self):
        new_email = self.cleaned_data['new_email']
        new_email_confirm = self.cleaned_data['new_email_confirm']

        if new_email != new_email_confirm:
            raise forms.ValidationError(
                self.error_messages['invalid_confirm_email'],
                code='invalid_confirm_email',
            )

        return new_email_confirm

    def send_email(self):
        """Sends the necessary emails and returns the ChangeEmail object. This
        method assumes the form has already made a call to ``.clean(...)``.
        """
        authorization = ChangeEmailAuthorization.objects.create(
            email_address=self.cleaned_data.get('new_email'),
            created_user=self.user
        )
        send_change_email_notice_email(
            to_user=self.user,
            authorization=authorization
        )
        send_change_email_activation_email(
            to_user=self.user,
            to_email=self.cleaned_data.get('new_email'),
            authorization=authorization
        )
        return authorization



from __future__ import unicode_literals

from django.conf.urls import patterns
from django.conf.urls import url

from .views import ActivateAccountExpiredView
from .views import ActivateAccountSuccessView
from .views import ActivateAccountView


urlpatterns = patterns('',
    url(r'^/activate/expired/?$', ActivateAccountExpiredView.as_view(), name='umanage_activate_account_expired'),
    url(r'^/activate/success/?$', ActivateAccountSuccessView.as_view(), name='umanage_activate_account_success'),
    url(r'^/activate/(?P<username>[\w.@+-]+)/(?P<authorization_token>(\w|-)+)/?$', ActivateAccountView.as_view(), name='umanage_activate_account'),
)



from __future__ import unicode_literals

from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import SetPasswordForm
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from django_core.utils.validators import is_valid_email
from django_core.utils.validators import validate_password_strength

from ..models import ForgotPasswordAuthorization
from .emails import send_forgot_password_email


class ForgotPasswordForm(forms.Form):
    """Form for forgot password."""
    username_or_email = forms.CharField(max_length=100)

    def clean_username_or_email(self):

        username_or_email = self.cleaned_data.get('username_or_email', '')
        username_or_email = username_or_email.strip()

        User = get_user_model()

        try:
            if is_valid_email(username_or_email):
                user = User.objects.get(email__iexact=username_or_email)
            else:
                user = User.objects.get(username__iexact=username_or_email)
        except User.DoesNotExist as e:
            raise ValidationError(_('No user found matching that username or '
                                    'email.'))

        self.user = user
        return username_or_email

    def send_email(self):
        """Sends the email for the forgot password flow"""
        authorization = ForgotPasswordAuthorization.objects.create(
            created_user=self.user
        )
        send_forgot_password_email(to_user=self.user,
                                   authorization=authorization)


class UManageSetPasswordForm(SetPasswordForm):

    next = forms.CharField(max_length=200, required=False,
                           widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        super(UManageSetPasswordForm, self).__init__(*args, **kwargs)
        self.fields['new_password1'].validators.append(validate_password_strength)



from django.conf.urls import include
from django.conf.urls import patterns
from django.conf.urls import url

from .views import TokenExpiredView


urlpatterns = patterns('',
    url(r'^/token-expired/?$', TokenExpiredView.as_view(), name='umanage_token_expired'),
    url(r'', include('umanage.accounts.urls')),
    url(r'', include('umanage.activate_account.urls')),
    url(r'', include('umanage.change_email.urls')),
    url(r'', include('umanage.change_password.urls')),
)



from __future__ import unicode_literals

from django.conf.urls import patterns
from django.conf.urls import url

from .views import ForgotPasswordChangePasswordView
from .views import ForgotPasswordChangeSuccessView
from .views import ForgotPasswordExpiredView
from .views import ForgotPasswordSentView
from .views import ForgotPasswordView


urlpatterns = patterns('',
    url(r'^forgot-password/?$', ForgotPasswordView.as_view(), name='umanage_forgot_password'),
    url(r'^forgot-password/expired/?$', ForgotPasswordExpiredView.as_view(), name='umanage_forgot_password_expired'),
    url(r'^forgot-password/sent/?$', ForgotPasswordSentView.as_view(), name='umanage_forgot_password_sent'),
    url(r'^forgot-password/success/?$', ForgotPasswordChangeSuccessView.as_view(), name='umanage_forgot_password_success'),
    url(r'^forgot-password/(?P<username>[\w.@+-]+)/(?P<authorization_token>(\w|-)+)/?$', ForgotPasswordChangePasswordView.as_view(), name='umanage_forgot_password_change_password'),
)



from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django_core.models import TokenAuthorization


class AccountActivationAuthorization(TokenAuthorization):
    """Model that handles the account authorization flow."""
    reason_default = 'ACCOUNT_ACTIVATION'

    class Meta:
        proxy = True

    def get_absolute_url(self):
        args = [self.created_user.username, self.token]
        return reverse('umanage_activate_account', args=args)


class ChangeEmailAuthorization(TokenAuthorization):
    """Model that handles the change email flow."""
    reason_default = 'CHANGE_EMAIL'

    class Meta:
        proxy = True

    def get_absolute_url(self):
        return reverse('umanage_change_email_activation', args=[self.token])


class ForgotPasswordAuthorization(TokenAuthorization):
    """Model that handles the forgot password flow."""
    reason_default = 'FORGOT_PASSWORD'

    class Meta:
        proxy = True

    def get_absolute_url(self):
        args = [self.created_user.username, self.token]
        return reverse('umanage_forgot_password_change_password', args=args)



from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.views.generic.base import TemplateView
from django.views.generic.edit import FormView

from .forms import ForgotUsernameForm
from django.shortcuts import redirect


class ForgotUsernameView(FormView):

    template_name = 'umanage/forgot_username/forgot_username.html'
    form_class = ForgotUsernameForm

    def get(self, request, *args, **kwargs):

        if request.user.is_authenticated():
            # User is authenticated, no need for this flow.
            return redirect('/')

        return super(ForgotUsernameView, self).get(request, *args, **kwargs)

    def form_valid(self, form):
        form.send_email()
        return super(ForgotUsernameView, self).form_valid(form)

    def get_success_url(self):
        return reverse('umanage_forgot_username_sent')


class ForgotUsernameSentView(TemplateView):

    template_name = 'umanage/forgot_username/forgot_username_sent.html'



from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.views.generic.base import TemplateView
from django.views.generic.base import View
from django.views.generic.edit import FormView
from django_core.auth.views import AuthorizationTokenRequiredViewMixin
from django_core.views.mixins.auth import LoginRequiredViewMixin

from ..models import ChangeEmailAuthorization
from .forms import ChangeEmailForm


class ChangeEmailView(LoginRequiredViewMixin, FormView):
    """View for changing a user's email."""

    template_name = 'umanage/change_email/change_email.html'
    form_class = ChangeEmailForm

    def get_form_kwargs(self):
        form_kwargs = super(ChangeEmailView, self).get_form_kwargs()
        form_kwargs['user'] = self.request.user
        return form_kwargs

    def form_valid(self, form):
        form.send_email()
        return super(ChangeEmailView, self).form_valid(form)

    def get_success_url(self):
        return reverse('umanage_change_email_sent')


class ChangeEmailSentView(LoginRequiredViewMixin, TemplateView):
    """View that confirms an email will be sent to the user to update the
    email address.
    """
    template_name = 'umanage/change_email/change_email_sent.html'


class ChangeEmailSuccessView(LoginRequiredViewMixin, TemplateView):
    """View that confirms an email has been updated successfully."""
    template_name = 'umanage/change_email/change_email_success.html'


class ChangeEmailActivationView(LoginRequiredViewMixin,
                                AuthorizationTokenRequiredViewMixin, View):

    authorization_class = ChangeEmailAuthorization

    def get(self, request, *args, **kwargs):

        # It's a valid change email object, update the user's email
        self.request.user.email = self.authorization.email_address
        self.request.user.save()

        # Expire the change email token so it's no longer valid.
        self.authorization.expire()
        return redirect('umanage_change_email_success')


class ChangeEmailExpiredView(LoginRequiredViewMixin, TemplateView):

    template_name = 'umanage/change_email/change_email_expired.html'



from __future__ import unicode_literals

from django.conf.urls import patterns
from django.conf.urls import url

from .views import LogoutView


urlpatterns = patterns('',
    url(r'^logout/?$', LogoutView.as_view(), name='umanage_logout'),
)



from __future__ import unicode_literals

from django.conf import settings

from umanage.utils.configuration import get_required_setting


def common(request):
    """Common settings to put in context."""
    template_path = get_required_setting('UMANAGE_BASE_TEMPLATE')

    if not request.user.is_authenticated():
        template_path = getattr(settings,
                                'UMANAGE_BASE_UNAUTHENTICATED_TEMPLATE',
                                template_path)

    return {
        'UMANAGE_BASE_TEMPLATE': template_path
    }



from __future__ import unicode_literals

from django.template import Library
from django_core.utils.loading import get_function_from_settings
from django.conf import settings


register = Library()


@register.filter
def render_umanage_form(form):
    """Renders a form based on settings ``UMANAGE_FORM_RENDERER``
    setting.  This allows users to plug in different 3rd party form rendering
    apps while being able to maintain a consistent look and feel across their
    site.

    The function must accept 1 argument which is the form to be rendered. No
    other args will be passed.

    For example,  if I want to use the
    `django-bootstrap-form <https://github.com/tzangms/django-bootstrap-form>`_
    app to render forms, I would provide the following setting to the template
    tag form rendering function::

        UMANAGE_FORM_RENDERER = 'bootstrapform.templatetags.bootstrap.bootstrap'

    Then all forms will render using the django-bootstrap-form library.  You
    can optionally provide the following strings that will render that form
    using table, paragraph or list tags::

        UMANAGE_FORM_RENDERER = 'as_p'     # render form using <p> tags
        UMANAGE_FORM_RENDERER = 'as_table' # render form using <table>
        UMANAGE_FORM_RENDERER = 'as_ul'    # render form using <ul>

    This will default to rending the form to however the form's ``__str__``
    method is defined.
    """
    form_renderer_key = 'UMANAGE_FORM_RENDERER'
    form_renderer_str = getattr(settings, form_renderer_key, None)

    if form_renderer_str and '.' in form_renderer_str:
        renderer_func = get_function_from_settings(form_renderer_key)
        return renderer_func(form) if renderer_func else form

    if form_renderer_str == 'as_table':
        return form.as_table()

    if form_renderer_str == 'as_ul':
        return form.as_ul()

    if form_renderer_str == 'as_p':
        return form.as_p()

    return form



from __future__ import unicode_literals

from django.utils.translation import ugettext as _

from ..utils.emails import umanage_send_email


def send_change_forgot_username_email(to_user,
                                      markdown_template='umanage/emails/forgot_username.md',
                                      **kwargs):
    """Sends an email to a user letting them know their username."""
    return umanage_send_email(
        to_user=to_user,
        subject=_('The Username You Requested'),
        markdown_template=markdown_template,
        **kwargs)



from __future__ import unicode_literals

from django.conf import settings
from umanage.exceptions import UManageRequiredSettingImproperlyConfigured


def get_required_setting(settings_key, default=None):
    """Gets a required setting or throws an UManageSettingImproperlyConfigured
    exception if no default value is provided.
    """
    try:
        if default is None:
            return getattr(settings, settings_key)

        return getattr(settings, settings_key, default)
    except AttributeError as e:
        raise UManageRequiredSettingImproperlyConfigured(settings_key=settings_key)



from django.conf.urls import patterns
from django.conf.urls import url

from .views import ForgotUsernameSentView
from .views import ForgotUsernameView


urlpatterns = patterns('',
    url(r'^forgot-username/?$', ForgotUsernameView.as_view(), name='umanage_forgot_username'),
    url(r'^forgot-username/sent/?$', ForgotUsernameSentView.as_view(), name='umanage_forgot_username_sent'),
)



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.http.response import Http404
from django.shortcuts import redirect
from django.views.generic.base import TemplateView
from django.views.generic.edit import FormView
from django_core.auth.views import AuthorizationTokenRequiredViewMixin

from ..models import ForgotPasswordAuthorization
from .forms import ForgotPasswordForm
from .forms import UManageSetPasswordForm


class ForgotPasswordView(FormView):

    template_name = 'umanage/forgot_password/forgot_password.html'
    form_class = ForgotPasswordForm

    def get(self, request, *args, **kwargs):

        if request.user.is_authenticated():
            return redirect('/')

        return super(ForgotPasswordView, self).get(request, *args, **kwargs)

    def form_valid(self, form):
        form.send_email()
        return super(ForgotPasswordView, self).form_valid(form)

    def get_success_url(self):
        return reverse('umanage_forgot_password_sent')


class ForgotPasswordSentView(TemplateView):

    template_name = 'umanage/forgot_password/forgot_password_sent.html'


class ForgotPasswordChangePasswordView(AuthorizationTokenRequiredViewMixin,
                                       FormView):

    template_name = 'umanage/forgot_password/forgot_password_change_password.html'
    form_class = UManageSetPasswordForm
    authorization_class = ForgotPasswordAuthorization

    def get_authorization_user(self):
        if self.authorization_user is not None:
            return self.authorization_user

        User = get_user_model()

        try:
            user = User.objects.get(username=self.kwargs.get('username'))
        except User.DoesNotExist as e:
            raise Http404

        self.authorization_user = user
        return user

    def get_auth_expired_url(self):
        return reverse('umanage_forgot_password_expired')

    def get_form_kwargs(self):
        kwargs = super(ForgotPasswordChangePasswordView,
                       self).get_form_kwargs()
        kwargs['user'] = self.get_authorization_user()
        return kwargs

    def form_valid(self, form):
        form.save()
        self.authorization.expire()
        return super(ForgotPasswordChangePasswordView, self).form_valid(form)

    def get_success_url(self):
        return reverse('umanage_forgot_password_success')


class ForgotPasswordExpiredView(TemplateView):

    template_name = 'umanage/forgot_password/forgot_password_expired.html'


class ForgotPasswordChangeSuccessView(TemplateView):

    template_name = 'umanage/forgot_password/forgot_password_change_success.html'



from __future__ import unicode_literals

from django.contrib.auth import logout
from django.shortcuts import redirect
from django.views.generic.base import TemplateView
from django_core.views.mixins.auth import LoginRequiredViewMixin
from django_core.views.mixins.csrf import CsrfExemptViewMixin


class LogoutView(CsrfExemptViewMixin, LoginRequiredViewMixin, TemplateView):

    template_name = 'umanage/auth/logout.html'

    def post(self, request, *args, **kwargs):
        logout(request)
        return redirect('/')



from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.http.response import HttpResponseForbidden
from django.views.generic.base import TemplateView
from django.views.generic.edit import FormView
from django_core.utils.urls import is_legit_next_url
from django_core.views.mixins.auth import LoginRequiredViewMixin

from ..forgot_password.forms import UManageSetPasswordForm
from .forms import UManagePasswordChangeForm


class ChangePasswordView(LoginRequiredViewMixin, FormView):
    """View for changing a user's password."""

    template_name = 'umanage/change_password/change_password.html'
    form_class = UManagePasswordChangeForm

    def get_form_kwargs(self):
        form_kwargs = super(ChangePasswordView, self).get_form_kwargs()
        form_kwargs['user'] = self.request.user
        return form_kwargs

    def form_valid(self, form):
        form.save()
        return super(ChangePasswordView, self).form_valid(form)

    def get_success_url(self):
        return reverse('umanage_change_password_success')


class ChangePasswordSuccessView(LoginRequiredViewMixin, TemplateView):
    """View for when a password has successfully been changed."""
    template_name = 'umanage/change_password/change_password_success.html'


class SetPasswordView(LoginRequiredViewMixin, FormView):
    """View for a user who has an unusable password and needs to set a
    password.  This can happen if a user authenticates through a 3rd party
    system and no usable password has been set.
    """
    template_name = 'umanage/change_password/set_password.html'
    form_class = UManageSetPasswordForm

    def get_form_kwargs(self):
        form_kwargs = super(SetPasswordView, self).get_form_kwargs()
        form_kwargs['user'] = self.request.user
        return form_kwargs

    def get_initial(self):
        initial = super(SetPasswordView, self).get_initial()
        initial['next'] = self.request.GET.get('next')
        return initial

    def post(self, request, *args, **kwargs):

        if request.user.has_usable_password():
            # this user already has a usable password and should not be posting
            # to this view.
            return HttpResponseForbidden()

        return super(SetPasswordView, self).post(request, *args, **kwargs)

    def form_valid(self, form):
        form.save()

        next_url = form.cleaned_data.get('next')
        if is_legit_next_url(next_url=next_url):
            self.success_url = next_url

        return super(SetPasswordView, self).form_valid(form)

    def get_success_url(self):
        return self.success_url or reverse('umanage_change_password_success')



from __future__ import unicode_literals

from django import forms
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from umanage.forgot_username.emails import send_change_forgot_username_email


class ForgotUsernameForm(forms.Form):
    """Form for forgot password."""
    email = forms.EmailField()

    def clean_email(self):
        email = self.cleaned_data.get('email')

        User = get_user_model()

        try:
            user = User.objects.get(email__iexact=email)
        except User.DoesNotExist as e:
            raise ValidationError(_('No user found with that email address.'))

        self.user = user
        return email

    def send_email(self):
        """Sends the email for the forgotten password."""
        send_change_forgot_username_email(to_user=self.user)



from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model
from django.http.response import Http404
from django.shortcuts import redirect
from django.views.generic.base import TemplateView
from django.views.generic.base import View
from django_core.auth.views import AuthorizationTokenRequiredViewMixin

from ..models import AccountActivationAuthorization


class ActivateAccountView(AuthorizationTokenRequiredViewMixin, View):

    authorization_class = AccountActivationAuthorization

    def dispatch(self, request, *args, **kwargs):
        user = self.get_authorization_user()

        if user and user.is_active:
            return redirect('/')

        return super(ActivateAccountView, self).dispatch(request,
                                                         *args,
                                                         **kwargs)

    def get(self, request, *args, **kwargs):
        self.authorization.created_user.is_active = True
        self.authorization.created_user.save()

        self.authorization.expire()
        return redirect('umanage_activate_account_success')

    def get_authorization_user(self):
        User = get_user_model()
        try:
            user = User.objects.get(username=self.kwargs.get('username'))
        except User.DoesNotExist as e:
            raise Http404

        self.authorization_user = user
        return user


class ActivateAccountSuccessView(TemplateView):

    template_name = 'umanage/activate_account/activate_account_success.html'

    def get_context_data(self, **kwargs):
        context = super(ActivateAccountSuccessView,
                         self).get_context_data(**kwargs)
        context['login_url'] = getattr(settings, 'LOGIN_URL', '/login')
        return context


class ActivateAccountExpiredView(TemplateView):

    template_name = 'umanage/activate_account/activate_account_expired.html'



from django.conf.urls import patterns
from django.conf.urls import url

from .views import ChangePasswordSuccessView
from .views import ChangePasswordView
from .views import SetPasswordView


urlpatterns = patterns('',
    url(r'^/change-password/?$', ChangePasswordView.as_view(), name='umanage_change_password'),
    url(r'^/change-password/success/?$', ChangePasswordSuccessView.as_view(), name='umanage_change_password_success'),
    url(r'^/set-password/?$', SetPasswordView.as_view(), name='umanage_set_password'),
)



from __future__ import unicode_literals

from django.conf.urls import patterns
from django.conf.urls import url

from .views import AccountEditView
from .views import AccountView


urlpatterns = patterns('',
    url(r'^/?$', AccountView.as_view(), name='umanage_account_view'),
    url(r'^/edit/?$', AccountEditView.as_view(), name='umanage_account_edit'),
)



from __future__ import unicode_literals

from django.conf import settings
from django.core.mail.message import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from markdown import markdown

from umanage.utils.configuration import get_required_setting


def umanage_send_email(to_user, subject, markdown_template=None,
                       text_template=None, html_template=None, to_email=None,
                       from_email=None, fail_silently=False, context=None):
    """Sends an email to a user.

    :param to_email: the email address to send the email to.  If an email needs
        to be sent to a user to a different email than what's currently
        registered (as does with the change email flow), the email will be sent
        to this email address.
    """
    if not to_user.email and not to_email:
        return None

    if not from_email:
        from_email = get_required_setting('UMANAGE_FROM_EMAIL')

    context = _get_email_context(context)
    context['to_user'] = to_user

    base_html_template = getattr(settings,
                                 'UMANAGE_BASE_HTML_TEMPLATE',
                                 'umanage/emails/base_email.html')

    context['umanage_base_html_email_template'] = base_html_template

    if markdown_template:
        text_content = render_to_string(markdown_template, context)
        context['email_content'] = markdown(text_content)
        html_content = render_to_string(base_html_template, context)
    else:
        text_content = render_to_string(text_template, context)
        html_content = render_to_string(html_template, context)

    text_content = mark_safe(text_content.strip())
    html_content = mark_safe(html_content.strip())

    msg = EmailMultiAlternatives(subject=subject,
                                 body=text_content,
                                 from_email=from_email,
                                 to=[to_email or to_user.email])
    msg.attach_alternative(html_content, 'text/html')
    msg.send()


def _get_email_context(context=None):
    if not context:
        context = {}

    context['site_root_uri'] = get_required_setting('UMANAGE_SITE_ROOT_URI')
    context['site_name'] = get_required_setting('UMANAGE_SITE_NAME')
    return context
